#!/bin/bash
#
# Author: patrick@cohesity.com
#
# Script to install or upgrade the AWS setup package. For first time installs,
# it creates the cohesity user and group and copies over the current user's
# ssh keys.
#
# Workflow for install or upgrade:
#   1. Download the installer package to /tmp on the VM.
#   2. Extract the package. There will be a tar.gz and this install.sh script.
#   3. Run the install.sh script with sudo privileges.
#   4. Remove the installer packages and script.

set -o nounset
set -o errexit
set -o pipefail

#set -x

SUDO="sudo"
COHESITY_GROUP="cohesity"
COHESITY_USER="cohesity"
COHESITY_ENCRYPTED_PASSWORD='$6$MESVKwtQ$2L0FSq9icgMAeMBVbVkXt0t9Jyj06w31ScC9LtTQW/m7IYp7oMR3ZKxz3CRs7UAObXWpv1okGhsduFJYIGliG0'
COHESITY_HOME="/home/$COHESITY_USER"
SOFTWARE_SYMLINK_NAME="software"
current_dir=$(dirname `readlink -f $0`)
current_time=$(date "+%Y.%m.%d-%H.%M.%S")
# Check that the script is called with sudo privileges.
user=$(whoami)
if [[ ${user} != "root" ]]; then
  echo "Must use sudo privileges"
  exit 1
fi

# Get the package name.
package_name=$(ls -1 ${current_dir}/cohesity_aws_setup*tar.gz)
num_pkgs=(${package_name})
if [[ ${#num_pkgs[@]} -gt 1 ]]; then
  echo -e "Expected one package. Found: \n${package_name}"
  exit 1
fi
echo "Found package ${package_name}"

# Get version string.
# cohesity_aws_setup-5.0_qa-20171009_17253c32.tar.gz -> 5.0_qa-20171009_17253c32
software_version=$(echo ${package_name} | cut -d"-" -f2- | sed 's/.tar.gz$//')
echo "Found software version ${software_version}"

# Read current software symlink if it exists.
is_upgrade=
old_software_version=
if [[ -L ${COHESITY_HOME}/${SOFTWARE_SYMLINK_NAME} ]]; then
  old_software_version=$(readlink ${COHESITY_HOME}/${SOFTWARE_SYMLINK_NAME})
  is_upgrade=true
  echo "Upgrading AWS setup package."
else
  echo "Installing AWS setup package."
fi

if [[ "${old_software_version}" == "${software_version}" ]]; then
  echo "AWS setup package version ${software_version} is already installed."
  exit 0
fi

# Create cohesity group if it doesn't exist.
match=$(grep --word-regexp ${COHESITY_GROUP} /etc/group || true)
if [[ ${match} == '' ]]; then
  ${SUDO} /usr/sbin/groupadd ${COHESITY_GROUP}
fi

# Create cohesity user if it doesn't exist.
match=$(grep --word-regexp ${COHESITY_USER} /etc/passwd || true)
if [[ ${match} == '' ]]; then
  ${SUDO} /usr/sbin/useradd -g ${COHESITY_GROUP} -m -d ${COHESITY_HOME} \
      -s /bin/bash ${COHESITY_USER} \
      -p ${COHESITY_ENCRYPTED_PASSWORD}
fi

# Add cohesity user to sudoers if not already added.
match=$(grep --word-regexp ${COHESITY_USER} /etc/sudoers || true)
if [[ ${match} == '' ]]; then
  ${SUDO} bash -lc 'echo cohesity ALL = NOPASSWD: ALL >> /etc/sudoers'
fi

# Make sure sshd config allows password authentication.
${SUDO} sed -i 's/^#PasswordAuthentication.*$/PasswordAuthentication yes/' /etc/ssh/sshd_config

# Update cloud.cfg to allow ssh password authentication.
${SUDO} sed -i 's/ssh_pwauth:.*0/ssh_pwauth:   1/' /etc/cloud/cloud.cfg || true

# Extract the package.
# Flag "skip-old-files" will preserve any existing data directory files that
# may conflict with files deployed by the package.
# Toolchain and bin files are deployed to a new software version directory, so
# they shouldn't have any old file conflicts.
# If an existing cluster_config_params.json exists, renaming the file.
if [ -f ${COHESITY_HOME}/data/aws/cluster_config_params.json ]; then
  mv ${COHESITY_HOME}/data/aws/cluster_config_params.json ${COHESITY_HOME}/data/aws/cluster_config_params${current_time}.json.bak
fi

${SUDO} tar --skip-old-files --warning=existing-file -C / -zxf ${package_name}
${SUDO} chown --recursive ${COHESITY_USER}:${COHESITY_GROUP} ${COHESITY_HOME}

# Change dir to /home/cohesity which should exist after extracting the package
cd ${COHESITY_HOME}

# Create/update software symlink.
ln -sfn ${software_version} ${SOFTWARE_SYMLINK_NAME}

# Update PATH to include cluster creation binaries.
${SUDO} echo 'export PATH=$PATH:$HOME/software/bin' >> ${COHESITY_HOME}/.bash_profile

# Remove the old software version if any.
if [[ ${is_upgrade} ]]; then
  if [[ "${old_software_version}" != "${software_version}" ]]; then
    rm -r ${old_software_version}
  fi
  echo "Done upgrading AWS setup package."
else
  echo "Done installing AWS setup package."
fi
